import subprocess
from jdeppy.Sentence import Sentence
from jdeppy.Bunsetsu import Bunsetsu
from jdeppy.Morph import Morph


class JDepP:
    def __init__(self, option=None):
        self.devnul = open('/dev/null', 'w')
        if option is None:
            self.option = []
        else:
            self.option = option.split(' ')

    def parse(self, sentence):
        proc_mecab = subprocess.Popen(['mecab'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=self.devnul)
        proc_jdepp = subprocess.Popen(['jdepp'], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=self.devnul)
        mecab_stdout = proc_mecab.communicate((sentence + '\n').encode())
        mecab_result = mecab_stdout[0]
        jdepp_stdout = proc_jdepp.communicate(mecab_result)
        jdepp_lines = jdepp_stdout[0].decode().split('\n')
        jdepp_results = []
        sentence = None
        bunsetsu = None
        for jdepp_line in jdepp_lines:
            if jdepp_line.startswith('# S-ID:'):
                if sentence:
                    jdepp_results.append(sentence)
                sentence_id = int(jdepp_line.split(';')[0].split(':')[1].replace(' ', ''))
                sentence = Sentence(s_id=sentence_id, blist=[])
                continue

            elif jdepp_line.startswith('*'):
                if bunsetsu:
                    sentence.append_bunsetsu(bunsetsu)
                b_id, dep_id = tuple([int(c.replace('D', '')) for c in jdepp_line.split(' ') if c != '*'])
                bunsetsu = Bunsetsu(b_id=b_id, dep_id=dep_id, mlist=[])
                continue

            elif jdepp_line == 'EOS':
                sentence.append_bunsetsu(bunsetsu)
                jdepp_results.append(sentence)
                break

            else:
                surf, args = tuple(jdepp_line.split('\t'))
                morph = Morph(surface=surf, feature=args.split(','))
                bunsetsu.append_morph(morph)
                continue

        return jdepp_results



class Sentence:
    s_id = None
    blist = []

    def __init__(self, s_id, blist):
        self.s_id = s_id
        self.blist = blist

    def append_bunsetsu(self, bnst):
        self.blist.append(bnst)


from setuptools import setup, find_packages


setup(
    name='jdeppy',
    version='1.0',
    description='Python wrapper for J.DepP',
    url='https://github.com/matsurihime/jdeppy',
    author='matsurihime',
    author_email='pipikapur@gmail.com',
    license='MIT',
    keywords='jdepp',
    # packages=find_packages(),
    classifiers=[
        'Natural Language :: Japanese',
    ],
)



class Bunsetsu:
    b_id = None
    dep_id = None
    mlist = []

    def __init__(self, b_id, dep_id, mlist):
        self.b_id = b_id
        self.dep_id = dep_id
        self.mlist = mlist

    def append_morph(self, mrph):
        self.mlist.append(mrph)
